from _typeshed import Incomplete
from typing import Any, Dict, List, Optional, Tuple

LOGGER: Incomplete

def prepare_file_name_saving(
    file_name: str,
    prefix: Optional[str | None] = None,
    suffix: Optional[str | None] = None,
) -> str: ...
def insert_string_in_file_name(file_name: str, insertion: str, suffix: str) -> str: ...
def make_full_filename(prefix: str, file_name: str) -> str: ...
def load_json_from_file(file_name: str) -> Dict[Any, Any]: ...
def save_json_to_file(
    dictionary: Dict[Any, Any], file_name: Optional[str] = ..., suffix: str = ...
) -> None: ...
def make_date_file_name(
    prefix: str = ..., file_name: str = ..., suffix: str = ...
) -> str: ...
def delete_file(file_name: str) -> None: ...
def search_subdirs(dir_name: str) -> Tuple[List[str], List[str]]: ...
def write_tar(target_dir: str, file_name: str) -> None: ...
def copy_dir(old_dir: str, new_dir: str, overwrite: bool = ...) -> None: ...
